//
//  PQCompositionExporter.swift
//  GPUImage_iOS
//
//  Created by ak on 2020/8/25.
//  Copyright © 2020 Sunset Lake Software LLC. All rights reserved.
//  功能： 导出视频

import AVFoundation
import Foundation
import ObjectMapper
// import GPUImage

public class PQCompositionExporter {
    public var progressClosure: ((_ currTime: Float, _ duration: Float, _ progress: Float) -> Void)?

    public var completion: ((_ url: URL) -> Void)?

    public let asset: AVAsset
    public let videoComposition: AVVideoComposition?
    public let audioMix: AVAudioMix?
    public let filters: [ImageProcessingOperation]?
    public var mStickers:[PQEditVisionTrackMaterialsModel]?
    public let animationTool: AVVideoCompositionCoreAnimationTool?
    public let exportURL: URL

    public var input: PQMoveInput?
    public var output: MovieOutput?

    public let tmpExportURL: URL?
    
    public var usedGroupCreateFilter:Bool = false
    
    //最后一次显示的 sticker
    public var lastshowSticker:PQEditVisionTrackMaterialsModel?
    
    //是否显示高斯
    public var showGaussianBlur:Bool = false

    public init(asset: AVAsset, videoComposition: AVVideoComposition? = nil, audioMix: AVAudioMix? = nil, filters: [ImageProcessingOperation]? = nil,stickers:[PQEditVisionTrackMaterialsModel]? = nil, animationTool: AVVideoCompositionCoreAnimationTool? = nil, exportURL: URL) {
        self.asset = asset
        self.videoComposition = videoComposition
        self.audioMix = audioMix
        self.filters = filters
        self.mStickers = stickers
        self.animationTool = animationTool
        self.exportURL = exportURL
        if self.animationTool != nil {
            tmpExportURL = URL(fileURLWithPath: String.qe.tmpPath() + String.qe.timestamp() + "_tmp.mp4")
        } else {
            tmpExportURL = nil
        }

    }

    public func prepare(videoSize: CGSize,videoAverageBitRate:Int = 0) -> Bool {
        guard !FileManager.default.fileExists(atPath: exportURL.absoluteString) else {
            return false
        }
        //        guard let videoTrack = asset.tracks(withMediaType:AVMediaType.video).first else { return false }
        let audioTrack = asset.tracks(withMediaType: AVMediaType.audio).first

        //
        //        Float64 seconds = 3;
        //        int32_t preferredTimeScale = 600;  //处理视频内容事常见的时间刻度为600，这是大部分视频帧率24fps、25fps、30fps的公倍数。音频数据常见的时间刻度就是采样率，譬如44 100(44.1kHZ)或48 000(48kHZ)。
        //        CMTime inTime = CMTimeMakeWithSeconds(seconds, preferredTimeScale);
        //        CMTimeShow(inTime);
        //
        // file:///private/var/mobile/Containers/Data/Application/9FDA94D5-D6E5-4DD7-A3C3-3D1D62318C33/tmp/1600948914.410534.m4a👈
        let audioDecodingSettings = [
            AVFormatIDKey: kAudioFormatLinearPCM,
            AVNumberOfChannelsKey: 1,
//               AVEncoderBitRateKey: 16000
        ]
        var acl = AudioChannelLayout()
        memset(&acl, 0, MemoryLayout<AudioChannelLayout>.size)
        acl.mChannelLayoutTag = kAudioChannelLayoutTag_Mono
        let audioEncodingSettings: [String: Any] = [
            AVFormatIDKey: kAudioFormatMPEG4AAC,
            AVNumberOfChannelsKey: 1,
            AVSampleRateKey: AVAudioSession.sharedInstance().sampleRate,
            AVChannelLayoutKey: NSData(bytes: &acl, length: MemoryLayout<AudioChannelLayout>.size),
            AVEncoderBitRateKey: 32000,
        ]

        do {
            if audioMix != nil {
                try input = PQMoveInput(asset: asset, videoComposition: nil, audioMix: audioMix, playAtActualSpeed: true, loop: false, audioSettings: audioDecodingSettings)
            } else {
                try input = PQMoveInput(asset: asset, videoComposition: nil, audioMix: nil, playAtActualSpeed: false, loop: false, audioSettings: audioDecodingSettings)
            }

            input?.mShowVidoSize = videoSize

        } catch {
            print("Couldn't process movie with error: \(error)")
        }
        BFLog(message: "export mShowVidoSize is \(String(describing: input?.mShowVidoSize))")
    
        let videoEncodingSettings: [String: Any] = [
            AVVideoCompressionPropertiesKey: [
                AVVideoExpectedSourceFrameRateKey: 30,
                AVVideoAverageBitRateKey: videoAverageBitRate == 0 ? Int(input?.mShowVidoSize.width ?? 0) * Int(input?.mShowVidoSize.height ?? 0) * 2 : videoAverageBitRate,
                AVVideoProfileLevelKey: AVVideoProfileLevelH264HighAutoLevel,
                AVVideoH264EntropyModeKey: AVVideoH264EntropyModeCABAC,
            ],
            //                AVVideoAllowFrameReorderingKey: videoTrack.requiresFrameReordering],
            AVVideoCodecKey: AVVideoCodecH264,
        ] // deprecated: 11.0, renamed: "AVVideoCodecType.h264" 改名

        do {
            try output = MovieOutput(URL: tmpExportURL ?? exportURL, size: Size(width: Float(input?.mShowVidoSize.width ?? 1080), height: Float(input?.mShowVidoSize.height ?? 1080)), fileType: .mp4, liveVideo: false, videoSettings: videoEncodingSettings, videoNaturalTimeScale: 30, audioSettings: audioEncodingSettings, audioSourceFormatHint: nil)
        } catch {
            return false
        }

        if audioTrack != nil {
            input?.audioEncodingTarget = output
        }
        input?.synchronizedMovieOutput = output
        
        if(mStickers == nil){
            var currentTarget: ImageSource = input!

            if filters?.count ?? 0 > 0 {
                filters!.forEach {
                    currentTarget.addTarget($0, atTargetIndex: 0)
                    currentTarget = $0
                }
            }

            currentTarget.addTarget(output!, atTargetIndex: 0)

        }
 
        return true
    }
    
   public  func  findShowStikcer(currTime:Float64)  {
         
        if(mStickers?.count ?? 0 == 0){
            BFLog(message: "mStickers data is error")
            return
        }
        var currentSticker:PQEditVisionTrackMaterialsModel?
        for sticker in mStickers! {
            BFLog(message: "sticker in \(sticker.timelineIn) out \(sticker.timelineOut)  currTime is \(currTime)")
            if(sticker.timelineIn <= currTime && sticker.timelineOut >= currTime){
                currentSticker = sticker
                break
            }
        }
        //创建不同的filter
        if(currentSticker == nil){
            BFLog(message: "sticker data is error")
            return
        }
        if(input != nil && lastshowSticker != currentSticker){
            var showFitler:PQBaseFilter?
            if currentSticker!.type == StickerType.VIDEO.rawValue {
                showFitler = PQMoveFilter(movieSticker: currentSticker!)
     

            } else if currentSticker!.type == StickerType.IMAGE.rawValue {
                showFitler = PQImageFilter(sticker: currentSticker!)
                
            }
            input!.removeAllTargets()
            let currentTarget: ImageSource = input!
            if(currentSticker?.type == StickerType.IMAGE.rawValue && showGaussianBlur){
                //高斯层
                let json = currentSticker?.toJSONString(prettyPrint: false)
                if json == nil {
                    BFLog(message: "数据转换有问题 跳转")
                    return
                }
                let blurStickerModel: PQEditVisionTrackMaterialsModel? = Mapper<PQEditVisionTrackMaterialsModel>().map(JSONString: json!)
                blurStickerModel?.canvasFillType = stickerContentMode.aspectFillStr.rawValue
                let showGaussianFitler:PQBaseFilter = PQImageFilter(sticker: blurStickerModel!)
                
                let iosb:GaussianBlur = GaussianBlur.init()
                iosb.blurRadiusInPixels = 20
                showGaussianFitler.addTarget(iosb)
                
                currentTarget.addTarget(showGaussianFitler, atTargetIndex: 0)
                
                iosb.addTarget(showFitler!)
     
                showFitler?.addTarget(output!, atTargetIndex: 0)
            }else{
                currentTarget.addTarget(showFitler!, atTargetIndex: 0)
                showFitler?.addTarget(output!, atTargetIndex: 0)

            }

            lastshowSticker = currentSticker
        }
 
    }

    public func start(playeTimeRange:CMTimeRange = CMTimeRange.init()) {
        input?.completion = { [unowned self] in
            self.output?.finishRecording { [unowned self] in
                BFLog(message: "导出视频完成发通知")
                DispatchQueue.main.async {
                    if let url = self.tmpExportURL {
                        self.handleCaption(for: AVURLAsset(url: url, options: nil))
                    } else {
                        self.progressClosure?(0, 0, 1)
                        self.completion?(self.exportURL)
                    }
                }
            }
        }

        input?.progress = { [weak self] currTime, duration, progress in
            
            if(self?.mStickers?.count ?? 0 > 0){
                self?.findShowStikcer(currTime: CMTimeGetSeconds(self?.input?.currentTime ?? .zero) )
            }
            DispatchQueue.main.async {[weak self] in
                self?.progressClosure?(Float(currTime), Float(duration), Float(progress))
            }
        }

        output?.startRecording { [unowned self] started, error in
            guard started else {
                NXLog(message: "MovieOutput unable to start writing with error: \(String(describing: error)) ")
                return
            }

            self.input?.start(isFreeBuffer: false, isExport: true,timeRange: playeTimeRange)
        }
    }

    // 取消输出
    public func cancel() {
        input?.cancel()
        output?.assetWriter.cancelWriting()
        output?.finishRecording()
    }

    public func handleCaption(for asset: AVAsset) {
        guard let animationTool = animationTool else { return }
        guard let assetVideoTrack = asset.tracks(withMediaType: .video).first else { return }
        guard let assetAudioTrack = asset.tracks(withMediaType: .audio).first else { return }
        let mixComposition = AVMutableComposition()
        guard let videoTrack = mixComposition.addMutableTrack(withMediaType: .video, preferredTrackID: kCMPersistentTrackID_Invalid) else { return }
        guard let audioTrack = mixComposition.addMutableTrack(withMediaType: .audio, preferredTrackID: kCMPersistentTrackID_Invalid) else { return }
        do {
            try videoTrack.insertTimeRange(CMTimeRange(start: .zero, duration: asset.duration), of: assetVideoTrack, at: .zero)
            try audioTrack.insertTimeRange(CMTimeRange(start: .zero, duration: asset.duration), of: assetAudioTrack, at: .zero)
        } catch {
            NXLog(message: "error is \(error)")
        }

        videoTrack.preferredTransform = assetVideoTrack.preferredTransform

        let size = videoTrack.naturalSize

        let videoComposition = AVMutableVideoComposition()
        videoComposition.renderSize = size
        videoComposition.animationTool = animationTool
        videoComposition.frameDuration = CMTime(value: 1, timescale: 60)
        let instruction = AVMutableVideoCompositionInstruction()
        instruction.timeRange = CMTimeRange(start: .zero, duration: asset.duration)
        let layerInstruction = AVMutableVideoCompositionLayerInstruction(assetTrack: videoTrack)
        instruction.layerInstructions = [layerInstruction]
        videoComposition.instructions = [instruction]

        let exporter = AVAssetExportSession(asset: mixComposition, presetName: AVAssetExportPresetHighestQuality)
        exporter?.outputURL = exportURL
        exporter?.outputFileType = .mp4
        exporter?.shouldOptimizeForNetworkUse = true
        exporter?.videoComposition = videoComposition
        exporter?.exportAsynchronously { [weak self] in
            guard let strongSelf = self else { return }
            BFLog(message: "导出完成 \(strongSelf.exportURL)")
            DispatchQueue.main.async {
                strongSelf.progressClosure?(1, 1, 1)
                strongSelf.completion?(strongSelf.exportURL)
            }
        }
    }
}
